unit Command;

interface

uses NTypes, NLib, NSys, NStr, NArgParser;

type
  ECommandError = class(Exception);

var
  Suspended: Boolean;
  MenuItems: array of TStringArray;

procedure DispatchCommand(const Args: array of string);

implementation

uses Windows, ShellAPI, NConf, NrLib, WinMan, Main, NotifyIcon;

procedure exec(const Args: array of string);
var
  a: TStringArray;
  dir: string;
  opt: Char;
begin
  dir := '';
  with ArgParser(Args) do
  begin
    while GetOpt(opt) do
      case opt of
        'd': dir := GetArg;
      end;
    a := Args;
  end;
  if Length(a) = 0 then
    raise ECommandError.Create('filename not specified');
  try
    if ShellExecute(GetDesktopWindow, nil, PChar(a[0]),
        PChar(JoinCommandLine(SliceArray(a, 1))), PChar(dir),
        SW_SHOWNORMAL) <= 32 then
      raise EOSError.Create;
  except
    on E: EOSError do raise ECommandError.Create(a[0]+': '+E.Msg);
  end;
end;

procedure menu(const Args: array of string);
var
  f: TFile;
  fwnd: HWND; 
  i: Integer;
  line: string;
  menu: HMENU;
  pt: TPoint;
  rc: TRect;
begin
  if Length(Args) = 0 then
    raise ECommandError.Create('filename not specified');
  menu := CreatePopupMenu;
  try
    try
      f := TFile.Open(JoinPath([ProfDir, Args[0]]));
      try
        i := 0;
        SetLength(MenuItems, 1024);
        while TConf.ReadLine(f, line) do
        begin
          if i = Length(MenuItems) then
            SetLength(MenuItems, i + i div 4);
          MenuItems[i] := SplitCommandLine(line);
          AppendMenu(menu, MF_STRING, IDM_MENUITEM_START + i,
              PChar(MenuItems[i][0]));
          Inc(i);
        end;
      finally
        f.Free;
      end;
      SystemParametersInfo(SPI_GETWORKAREA, 0, @rc, 0);
      // to hold mouse cursor position
      GetCursorPos(pt);
      SetWindowPos(MainWnd, 0, pt.x, pt.y, 0, 0, SWP_NOSIZE or SWP_NOZORDER);
      fwnd := GetForegroundWindow;
      SetForegroundWindow(MainWnd);
      TrackPopupMenu(menu,
        TPM_CENTERALIGN or TPM_VCENTERALIGN or TPM_LEFTBUTTON or
          TPM_RIGHTBUTTON,
        (rc.Right-rc.Left) div 2, (rc.Bottom-rc.Top) div 2, 0, MainWnd, nil);
      if GetForegroundWindow = MainWnd then
        SetForegroundWindow(fwnd);
    except
      on E: EIOError do raise ECommandError.Create(E.Path+': '+E.Msg);
    end;
  finally
    DestroyMenu(menu);
  end;
end;

procedure suspend(const Args: array of string);
begin
  Suspended := not Suspended;
  if Suspended then
  begin
    SuspendHotKeys;
    SetNotifyIconState(True);
  end
  else begin
    UnsuspendHotKeys;
    SetNotifyIconState(False);
  end;
end;

procedure window(const Args: array of string);
var
  i, x, y, w, h: Longint;
  origin: TPositionOrigin;
  rflag: Boolean;
  opt: Char;
  title, classname, arg: string;
  wnd: HWND;
begin
  if Length(Args) = 0 then
    raise ECommandError.Create('command not specified');

  if Args[0] = 'foreground' then
  begin
    i := 1;
    rflag := False;
    if (Length(Args) > 1) and (Args[1] = '-r') then
    begin
      rflag := True;
      i := 2;
    end;
    if Length(Args) > i then
      Foreground(Args[i], rflag);
  end
  else if Args[0] = 'info' then
  begin
    GetInfo(title, wnd, classname, x, y, w, h);
    MsgBox(Format(
        'Window Name: {0}'#10+
        'Window Handle: {1}'#10+
        'Class Name: {2}'#10+
        'Position: x={3} y={4} w={5} h={6}',
        [title, wnd, classname, x, y, w, h]));
  end
  else if Args[0] = 'minimize' then
    Minimize
  else if Args[0] = 'minimizeall' then
    MinimizeAll
  else if Args[0] = 'pos' then
  begin
    x := spUndef;
    y := spUndef;
    w := spUndef;
    h := spUndef;
    origin := poLeftTop;
    with ArgParser(SliceArray(Args, 1)) do
      while GetOpt(opt) do
        case opt of
          'x': x := SInt(GetArg);
          'y': y := SInt(GetArg);
          'w': w := SInt(GetArg);
          'h': h := SInt(GetArg);
          'o': begin
                 arg := GetArg;
                 if arg = 'lb' then origin := poLeftBottom
                 else if arg = 'rt' then origin := poRightTop
                 else if arg = 'rb' then origin := poRightBottom
               end;
        end;
    SetPosition(x, y, w, h, origin);
  end else
    raise ECommandError.Create('unknown command ''' + Args[0] + '''');
end;

function GetValue(const S: string): TStringArray;
var
  rc: TRect;
begin
  if S[1] = '$' then
  begin
    if S = '$_foregroundwindow' then
      Result := StringArray([IStr(GetForegroundWindow)])
    else if S = '$_workareaheight' then
    begin
      SystemParametersInfo(SPI_GETWORKAREA, 0, @rc, 0);
      Result := StringArray([IStr(rc.Bottom - rc.Top)]);
    end
    else if S = '$_workareawidth' then
    begin
      SystemParametersInfo(SPI_GETWORKAREA, 0, @rc, 0);
      Result := StringArray([IStr(rc.Right - rc.Left)]);
    end else
      Result := StringArray([GetEnv(Copy(S, 2, Length(S)-1))]);
  end else
    Result := StringArray([S]);
end;

procedure DispatchCommand(const Args: array of string);
type
  TCommandProc = procedure (const Args: array of string);
  TCommand = record
    Name: string;
    Proc: TCommandProc;
  end;
const
  Commands: array[0..3] of TCommand = (
    (Name: 'exec'; Proc: exec),
    (Name: 'menu'; Proc: menu),
    (Name: 'suspend'; Proc: suspend),
    (Name: 'window'; Proc: window)
    );
var
  i: Integer;
  cmdproc: TCommandProc;
  expandedargs: TStringArray;
begin
  expandedargs := Expand(Args, GetValue);
  cmdproc := nil;
  for i := 0 to High(Commands) do
    if Commands[i].Name = expandedargs[0] then
    begin
      cmdproc := Commands[i].Proc;
      Break;
    end;
  if not Assigned(cmdproc) then
  begin
    MsgBox('unknown command '''+expandedargs[0]+'''');
    Exit;
  end;
  try
    cmdproc(SliceArray(expandedargs, 1));
  except
    on E: ECommandError do MsgBox(expandedargs[0]+': '+E.Msg);
  end
end;

end.
